@@ -1,5 +1,25 @@
+/*  Glimmer - bindings-python.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+#include "declarations.h"
+
 #include "bindings.h"
+#include "settings.h"
 #include "signals.h"
+#include "toolbar.h"
 
 static PyObject *_wrap_get_file_number(PyObject *self, PyObject *args)
 {
@@ -1132,167 +1152,184 @@ static PyObject *_wrap_add_embedded_entry(PyObject *self, PyObject *args)
    char *arg2;
    char *arg3;
    char *arg4;
-   char *arg5;
 
    self = self;
-   if(!PyArg_ParseTuple(args,"ssssss:add_embedded_entry",&arg0,&arg1,&arg2,&arg3,&arg4,&arg5)) 
+   if(!PyArg_ParseTuple(args,"ssssss:add_embedded_entry",&arg0,&arg1,&arg2,&arg3,&arg4)) 
       return NULL;
-   add_embedded_entry(arg0,arg1,arg2,arg3,arg4,arg5);
+   add_embedded_entry(arg0,arg1,arg2,arg3,arg4);
    Py_INCREF(Py_None);
    resultobj = Py_None;
    return resultobj;
 }
 
-static PyObject *add_item_to_scripts_menu(PyObject* self, PyObject* args)
-{
-   PyObject *callback;
-   GtkWidget *menu_item;
-   char* label;
-   int position;
-   GtkWidget *parent;
-   if(!PyArg_ParseTuple(args, "siO", &label, &position, &callback))
-    return NULL;
-   if(!PyCallable_Check(callback))
-   {
-     PyErr_SetString(PyExc_TypeError, "add_to_menu argument must be callable");
-     return NULL;
-   }
-   Py_INCREF(callback);
-   menu_item = gtk_menu_item_new_with_label(label);
-   parent = (GtkWidget *)g_list_nth_data(dynamic_scripts, position);
-   if(!parent || !GTK_IS_MENU(parent)) parent = scriptsmenu;
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(call_python_function_cb), (gpointer)callback);
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(remove_pyref), (gpointer)callback);
-   dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)menu_item);
-   gtk_widget_show(menu_item);
-   Py_INCREF(Py_None);
-   return Py_None;
-}
-
-static PyObject *add_sub_to_scripts_menu(PyObject* self,PyObject* args)
-{
-   PyObject *result;
-   GtkWidget *menu_item;
-   GtkWidget *submenu;
-   GtkWidget *tearoff;
-   char* label;
-   int position;
-   GtkWidget *parent;
-   if(!PyArg_ParseTuple(args, "si", &label, &position))
-     return NULL;
-   menu_item = gtk_menu_item_new_with_label(label);
-   dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)menu_item);
-   gtk_widget_show(menu_item);
-   parent = (GtkWidget *)g_list_nth_data(dynamic_scripts, position);
-   if(!parent || !GTK_IS_MENU(parent)) parent = scriptsmenu;
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   submenu = gtk_menu_new();
-   tearoff = gtk_tearoff_menu_item_new();
-   gtk_widget_show(tearoff);
-   gtk_menu_append(GTK_MENU(submenu), tearoff);
-   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);   
-   dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)submenu);
-   gtk_widget_show(submenu);
-   position = g_list_index(dynamic_scripts, (gpointer)submenu);
-   result = Py_BuildValue("i", position);
-   return result;
-}
-
-static PyObject *add_item_to_insert_menu(PyObject* self, PyObject* args)
+static PyObject *add_item_to_menu(PyObject* self, PyObject* args)
 {
    PyObject *callback;
    GtkWidget *menu_item;
+   char *path;
    char *label;
+   char *hint;
+   char *keys;
    int position;
-   GtkWidget *parent;
-   GList *ptr;
-   if(!PyArg_ParseTuple(args, "siO", &label, &position, &callback))
-    return NULL;
-   if(!PyCallable_Check(callback))
+   gchar *str;
+   GList *ptr = NULL;
+   gboolean separator;
+   GnomeUIInfo new_menu[] =
    {
-     PyErr_SetString(PyExc_TypeError, "add_to_menu argument must be callable");
-     return NULL;
+      GNOMEUIINFO_END,
+      GNOMEUIINFO_END
+   };
+   if(!PyArg_ParseTuple(args, "ssssiO", &path, &label, &hint, &keys, &position, &callback))
+      return NULL;
+   separator = label && strlen(label) ? FALSE : TRUE;
+   if(!separator && !PyCallable_Check(callback))
+   {
+      PyErr_SetString(PyExc_TypeError, "add_item_to_menu argument must be callable");
+      return NULL;
+   }
+   if(!separator) Py_INCREF(callback);
+	new_menu[0].type = separator ? GNOME_APP_UI_SEPARATOR : GNOME_APP_UI_ITEM;
+	new_menu[0].label = label;
+	new_menu[0].hint = hint;
+	new_menu[0].moreinfo = (gpointer)call_python_function_cb;
+	new_menu[0].user_data = (gpointer)callback;
+	new_menu[0].unused_data = NULL;
+	new_menu[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
+	new_menu[0].pixmap_info = NULL;
+   if (keys && strlen(keys))
+   {
+      gtk_accelerator_parse(keys, &new_menu[0].accelerator_key, &new_menu[0].ac_mods);
    }
-   Py_INCREF(callback);
-   menu_item = gtk_menu_item_new_with_label(label);
-   if(parse_table)
-     parent = (GtkWidget *)g_list_nth_data(parse_table->menu_entries, position);
    else
-     parent = (GtkWidget *)g_list_nth_data(static_entries, position);
-   if(!parent || !GTK_IS_MENU(parent))
-     parent = insertmenu;
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(call_python_function_cb), (gpointer)callback);
+   {
+      new_menu[0].accelerator_key = 0;
+      new_menu[0].ac_mods = (GdkModifierType) 0;
+   }
+	new_menu[0].widget = NULL;
+
+   gnome_app_insert_menus(GNOME_APP(app), path, &new_menu[0]);
+   menu_item = new_menu[0].widget;
+   gtk_menu_reorder_child(GTK_MENU(menu_item->parent), menu_item, position);
    gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(remove_pyref), (gpointer)callback);
-   if(parse_table)
+   if((str = strstr(path, "Scripts")) && str == path)
    {
-     parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
-     ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
+      dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)menu_item);
    }
    else
    {
-     static_entries = g_list_append(static_entries, (gpointer)menu_item);
-     ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
+      if(parse_table)
+      {
+         parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
+         ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
+      }
+      else
+      {
+         static_entries = g_list_append(static_entries, (gpointer)menu_item);
+         ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
+      }
    }
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(menu_item);
+   if(ptr) gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
    Py_INCREF(Py_None);
    return Py_None;
 }
 
-static PyObject *add_sub_to_insert_menu(PyObject* self,PyObject* args)
+static PyObject *add_sub_to_menu(PyObject* self,PyObject* args)
 {
-   PyObject *result;
-   GtkWidget *menu_item;
-   GtkWidget *submenu;
-   GtkWidget *tearoff;
+   GtkWidget *menu_item = NULL;
+   GtkWidget *submenu = NULL;
+   char *path;
    char* label;
-   int position = -1;
+   int position;
+   gchar *str;
+   GList *ptr = NULL;
+
+   GnomeUIInfo new_menu[] =
+   {
+      GNOMEUIINFO_END,
+      GNOMEUIINFO_END
+   };
+   GnomeUIInfo sub_menu[] =
+   {
+      GNOMEUIINFO_SEPARATOR,
+      GNOMEUIINFO_END
+   };
+
+   if(!PyArg_ParseTuple(args, "ssi", &path, &label, &position))
+      return NULL;
+	new_menu[0].type = GNOME_APP_UI_SUBTREE;
+	new_menu[0].label = label;
+	new_menu[0].hint = "";
+	new_menu[0].moreinfo = (gpointer)sub_menu;
+	new_menu[0].user_data = NULL;
+	new_menu[0].unused_data = NULL;
+	new_menu[0].pixmap_type = GNOME_APP_PIXMAP_NONE;
+	new_menu[0].pixmap_info = NULL;
+   new_menu[0].accelerator_key = 0;
+   new_menu[0].ac_mods = (GdkModifierType) 0;
+	new_menu[0].widget = NULL;
+
+   gnome_app_insert_menus(GNOME_APP(app), path, &new_menu[0]);
+   menu_item = new_menu[0].widget;
+   if(sub_menu[0].widget) submenu = sub_menu[0].widget->parent;
+   gtk_menu_reorder_child(GTK_MENU(menu_item->parent), menu_item, position);
+   
+   if((str = strstr(path, "Scripts")) && str == path)
+   {
+      dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)menu_item);
+      dynamic_scripts = g_list_append(dynamic_scripts, (gpointer)submenu);
+   }
+   else
+   {
+      if(parse_table)
+      {
+         parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
+         ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
+         if(ptr) gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
+         parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)submenu);
+         ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)submenu));
+         if(ptr) gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
+      }
+      else
+      {
+         static_entries = g_list_append(static_entries, (gpointer)menu_item);
+         ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
+         if(ptr) gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
+         static_entries = g_list_append(static_entries, (gpointer)submenu);
+         ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)submenu));
+         if(ptr) gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
+      }
+      if((str = strstr(path, "Build")) && str == path && parsing_language)
+         dynamic_build = g_list_append(dynamic_build, (gpointer)menu_item);
+   }
+   Py_INCREF(Py_None);
+   return Py_None;
+}
+
+static PyObject *add_widget_to_menu(PyObject* self, PyObject* args)
+{
+   PyObject *item;
+   GtkObject *obj;
    GtkWidget *parent;
-   GList *ptr;
-   if(!PyArg_ParseTuple(args, "si", &label, &position))
-     return NULL;
-   menu_item = gtk_menu_item_new_with_label(label);
-   if(parse_table)
+   GtkWidget *menu_item;
+   char *path;
+   gint pos = 0;
+   if(!PyArg_ParseTuple(args, "sO", &path, &item))
+      return NULL;
+   if(!item) return NULL;
+   if(!PyGtk_Check(item))
    {
-     parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
-     ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
+      g_print("add_widget_to_menu: Second argument must be a GtkObject!\n");
    }
    else
    {
-     static_entries = g_list_append(static_entries, (gpointer)menu_item);
-     ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
+      obj = PyGtk_Get(item);
+      if(!GTK_IS_WIDGET(obj)) return NULL;
+      menu_item = GTK_WIDGET(obj);
+      parent = gnome_app_find_menu_pos(GNOME_APP(app)->menubar, path, &pos);
+      if(parent) gtk_menu_shell_insert(GTK_MENU_SHELL(parent), menu_item, pos);
    }
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(menu_item);
-   if(parse_table)
-     parent = (GtkWidget *)g_list_nth_data(parse_table->menu_entries, position);
-   else
-     parent = (GtkWidget *)g_list_nth_data(static_entries, position);
-   if(!parent || !GTK_IS_MENU(parent)) parent = insertmenu;
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   submenu = gtk_menu_new();
-   tearoff = gtk_tearoff_menu_item_new();
-   gtk_menu_append(GTK_MENU(submenu), tearoff);
-   gtk_widget_show(tearoff);
-   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);   
-   if(parse_table)
-   {
-     parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)submenu);
-     position = g_list_index(parse_table->menu_entries, (gpointer)submenu);
-     ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)submenu));
-   }
-   else
-   {
-     static_entries = g_list_append(static_entries, (gpointer)submenu);
-     position = g_list_index(static_entries, (gpointer)submenu);
-     ptr = g_list_nth(static_entries, position);
-   }
-   gtk_signal_connect(GTK_OBJECT(submenu), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(submenu);
-   result = Py_BuildValue("i", position);
-   return result;
+   Py_INCREF(Py_None);
+   return Py_None;
 }
 
 static PyObject *add_item_to_popup_menu(PyObject* self, PyObject* args)
@@ -1307,7 +1344,7 @@ static PyObject *add_item_to_popup_menu(PyObject* self, PyObject* args)
     return NULL;
    if(!PyCallable_Check(callback))
    {
-     PyErr_SetString(PyExc_TypeError, "add_to_menu argument must be callable");
+     PyErr_SetString(PyExc_TypeError, "add_item_to_popup argument must be callable");
      return NULL;
    }
    Py_INCREF(callback); /* possible referennce leak */
@@ -1396,111 +1433,6 @@ static PyObject *add_sub_to_popup_menu(PyObject* self, PyObject* args)
    return result;
 }
 
-static PyObject *add_item_to_build_menu(PyObject* self, PyObject* args)
-{
-   PyObject *callback;
-   GtkWidget* menu_item;
-   char* label;
-   int position;
-   GtkWidget *parent;
-   GList *ptr;
-   if(!PyArg_ParseTuple(args, "siO", &label, &position, &callback))
-    return NULL;
-   if(!PyCallable_Check(callback))
-   {
-     PyErr_SetString(PyExc_TypeError, "add_to_menu argument must be callable");
-     return NULL;
-   }
-   Py_INCREF(callback); /* possible referennce leak */
-   menu_item = gtk_menu_item_new_with_label(label);
-   if(parse_table)
-     parent = (GtkWidget *)g_list_nth_data(parse_table->menu_entries, position);
-   else
-     parent = (GtkWidget *)g_list_nth_data(static_entries, position);
-   if(!parent || !GTK_IS_MENU(parent))
-   {
-     parent = buildmenu;
-     if(parsing_language)
-       dynamic_build = g_list_append(dynamic_build, (gpointer)menu_item);
-   }
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(call_python_function_cb), (gpointer)callback);
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(remove_pyref), (gpointer)callback);
-   if(parse_table)
-   {
-     parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
-     ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
-   }
-   else
-   {
-     static_entries = g_list_append(static_entries, (gpointer)menu_item);
-     ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
-   }
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(menu_item);
-   Py_INCREF(Py_None);
-   return Py_None;
-}
-
-static PyObject *add_sub_to_build_menu(PyObject* self,PyObject* args)
-{
-   PyObject *result;
-   GtkWidget *menu_item;
-   GtkWidget *submenu;
-   GtkWidget *tearoff;
-   char* label;
-   int position = -1;
-   GtkWidget *parent;
-   GList *ptr;
-   if(!PyArg_ParseTuple(args, "si", &label, &position))
-     return NULL;
-   menu_item = gtk_menu_item_new_with_label(label);
-   if(parse_table)
-   {
-     parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)menu_item);
-     ptr = g_list_nth(parse_table->menu_entries, g_list_index(parse_table->menu_entries, (gpointer)menu_item));
-   }
-   else
-   {
-     static_entries = g_list_append(static_entries, (gpointer)menu_item);
-     ptr = g_list_nth(static_entries, g_list_index(static_entries, (gpointer)menu_item));
-   }
-   gtk_signal_connect(GTK_OBJECT(menu_item), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(menu_item);
-   if(parse_table)
-     parent = (GtkWidget *)g_list_nth_data(parse_table->menu_entries, position);
-   else
-     parent = (GtkWidget *)g_list_nth_data(static_entries, position);
-   if(!parent || !GTK_IS_MENU(parent))
-   {
-     parent = buildmenu;
-     if(parsing_language)
-       dynamic_build = g_list_append(dynamic_build, (gpointer)menu_item);
-   }
-   gtk_menu_append(GTK_MENU(parent), menu_item);
-   submenu = gtk_menu_new();
-   tearoff = gtk_tearoff_menu_item_new();
-   gtk_menu_append(GTK_MENU(submenu), tearoff);
-   gtk_widget_show(tearoff);
-   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);   
-   if(parse_table)
-   {
-      parse_table->menu_entries = g_list_append(parse_table->menu_entries, (gpointer)submenu);
-      position = g_list_index(parse_table->menu_entries, (gpointer)submenu);
-      ptr = g_list_nth(parse_table->menu_entries, position);
-   }
-   else
-   {
-      static_entries = g_list_append(static_entries, (gpointer)submenu);
-      position = g_list_index(static_entries, (gpointer)submenu);
-      ptr = g_list_nth(static_entries, position);
-   }
-   gtk_signal_connect(GTK_OBJECT(submenu), "destroy", GTK_SIGNAL_FUNC(kill_widget_ptr), &ptr->data);
-   gtk_widget_show(submenu);
-   result = Py_BuildValue("i", position);
-   return result;
-}
-
 static PyObject *register_signal_hook(PyObject* self, PyObject* args)
 {
    PyObject *callback;
@@ -1551,6 +1483,17 @@ static PyObject *register_signal_hook(PyObject* self, PyObject* args)
          parse_table->tab_pressed_callback = (gpointer)callback;
          matched = TRUE;
       }
+      else if(!strcmp("anti-tab-pressed", name))
+      {
+         parse_table->anti_tab_pressed_callback = (gpointer)callback;
+         matched = TRUE;
+      }
+      else if(!strcmp("key-pressed", name))
+      {
+         g_print("key-pressed callback added.\n");
+         parse_table->key_pressed_callback = (gpointer)callback;
+         matched = TRUE;
+      }
       else if(!strcmp("compile", name))
       {
          parse_table->compile_hook = (gpointer)callback;
@@ -1738,7 +1681,6 @@ static PyObject *append_custom_toolbar_page(PyObject* self, PyObject* args)
    return result;
 }
 
-#ifdef HAVE_PYGTK
 static PyObject *add_dock_item(PyObject* self, PyObject* args)
 {
    PyObject *w;
@@ -1822,27 +1764,187 @@ static PyObject *get_text_widget(PyObject* self, PyObject* args)
    pyobj = PyGtk_New(obj);
    return(pyobj);
 }
-#endif
+
+static PyObject *add_paned_object(PyObject* self, PyObject* args)
+{
+   PyObject *w;
+   GtkObject *obj;
+   GtkWidget *widget;
+   GtkWidget *label;
+   GtkWidget *notebook;
+   GtkWidget **nbook;
+   GtkWidget *paned;
+   char *str;
+   int pos = 0;
+
+   if(!PyArg_ParseTuple(args, "Osi:add_paned_object", &w, &str, &pos))
+      return NULL;
+   if(!w)
+      return NULL;
+   obj = PyGtk_Get(w);
+   if(!GTK_IS_WIDGET(obj))
+      return NULL;
+   widget = GTK_WIDGET(obj);
+   switch(pos)
+   {
+      case 0:
+         nbook = &panedbook1;
+         notebook = panedbook1;
+         paned = hpaned1;
+         break;
+      case 1:
+         nbook = &panedbook2;
+         notebook = panedbook2;
+         paned = vpaned2;
+         break;
+      case 2:
+         nbook = &panedbook3;
+         notebook = panedbook3;
+         paned = hpaned3;
+         break;
+      default:
+         nbook = &panedbook4;
+         notebook = panedbook4;
+         paned = vpaned4;
+         break;
+   }
+
+   if(!notebook)
+   {
+      notebook = gtk_notebook_new();
+      *nbook = notebook;
+      if(paned == hpaned1 || paned == vpaned2)
+         gtk_paned_pack1(GTK_PANED(paned), notebook, TRUE, TRUE);
+      else
+         gtk_paned_pack2(GTK_PANED(paned), notebook, TRUE, TRUE);
+      gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);
+      gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+   }
+
+   label = gtk_label_new(str);
+   gtk_widget_show(label);
+   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), widget, label);
+   g_hash_table_insert(child_hash, str, (gpointer) widget);
+   gtk_widget_show(notebook);
+   if(g_list_length(GTK_NOTEBOOK(notebook)->children) > 1)
+      gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);
+   GTK_PANED(paned)->position_set = 0;
+   gtk_paned_set_gutter_size(GTK_PANED(paned), 8);
+   gtk_paned_set_handle_size(GTK_PANED(paned), 6);
+
+   adjust_sensitivity();
+   Py_INCREF(Py_None);
+   return(Py_None);
+}
+
+static PyObject *remove_paned_object(PyObject* self, PyObject* args)
+{
+   GtkWidget *child = NULL;
+   GtkNotebook *notebook = NULL;
+   GtkPaned *paned = NULL;
+   char *str = NULL;
+
+   if(!PyArg_ParseTuple(args, "s:remove_paned_object", &str))
+      return NULL;
+
+   child = g_hash_table_lookup(child_hash, str);
+   if(child)
+   {
+      notebook = GTK_NOTEBOOK(child->parent);
+      gtk_notebook_remove_page(notebook, gtk_notebook_page_num(notebook, child));
+      if(!g_list_length(notebook->children))
+      {
+         paned = GTK_PANED(GTK_WIDGET(notebook)->parent);
+         gtk_widget_hide(GTK_WIDGET(notebook));
+         gtk_paned_set_gutter_size(paned, 0);
+         gtk_paned_set_handle_size(paned, 0);
+         gtk_paned_set_position(paned, 0);
+      }
+      g_hash_table_remove(child_hash, str);
+   }
+   adjust_sensitivity();
+   Py_INCREF(Py_None);
+   return(Py_None);
+}
+
+static PyObject *get_paned_object(PyObject* self, PyObject* args)
+{
+   PyObject *pyobj = NULL;
+   GtkObject *obj = NULL;
+   char *str = NULL;
+
+   if(!PyArg_ParseTuple(args, "s:get_paned_object", &str))
+      return NULL;
+
+   obj = g_hash_table_lookup(child_hash, str);
+
+   if(obj)
+   {
+      pyobj = PyGtk_New(obj);
+      return(pyobj);
+   }
+   else
+   {
+      Py_INCREF(Py_None);
+      return(Py_None);
+   }
+}
+
+static PyObject *ref_paned_object(PyObject* self, PyObject* args)
+{
+   GtkObject *obj = NULL;
+   char *str = NULL;
+
+   if(!PyArg_ParseTuple(args, "s:ref_paned_object", &str))
+      return NULL;
+
+   obj = g_hash_table_lookup(child_hash, str);
+
+   if(obj)
+   {
+      gtk_object_ref(obj);
+   }
+   Py_INCREF(Py_None);
+   return(Py_None);
+}
+
+static PyObject *unref_paned_object(PyObject* self, PyObject* args)
+{
+   GtkObject *obj = NULL;
+   char *str = NULL;
+
+   if(!PyArg_ParseTuple(args, "s:unref_paned_object", &str))
+      return NULL;
+
+   obj = g_hash_table_lookup(child_hash, str);
+
+   if(obj)
+   {
+      gtk_object_unref(obj);
+   }
+   Py_INCREF(Py_None);
+   return(Py_None);
+}
 
 static PyMethodDef glimmerMethods[] =
 {
-#ifdef HAVE_PYGTK
+   { "unref_paned_object", unref_paned_object, 1 },
+   { "ref_paned_object", ref_paned_object, 1 },
+   { "get_paned_object", get_paned_object, 1 },
+   { "remove_paned_object", remove_paned_object, 1 },
+   { "add_paned_object", add_paned_object, 1 },
    { "get_text_widget", get_text_widget, 1 },
    { "get_file_box", get_file_box, 1 },
    { "get_gnome_app", get_gnome_app, 1 },
    { "add_dock_item", add_dock_item, 1 },
-#endif
    { "add_custom_toolbar_entry", add_custom_toolbar_entry, 1 },
    { "append_custom_toolbar_page", append_custom_toolbar_page, 1 },
    { "register_signal_hook", register_signal_hook, 1 },
-   { "add_item_to_build_menu", add_item_to_build_menu, 1 },
-   { "add_sub_to_build_menu", add_sub_to_build_menu, 1 },
    { "add_item_to_popup_menu", add_item_to_popup_menu, 1 },
    { "add_sub_to_popup_menu", add_sub_to_popup_menu, 1 },
-   { "add_item_to_insert_menu", add_item_to_insert_menu, 1 },
-   { "add_sub_to_insert_menu", add_sub_to_insert_menu, 1 },
-   { "add_item_to_scripts_menu", add_item_to_scripts_menu, 1 },
-   { "add_sub_to_scripts_menu", add_sub_to_scripts_menu, 1 },
+   { "add_widget_to_menu", add_widget_to_menu, 1 },
+   { "add_item_to_menu", add_item_to_menu, 1 },
+   { "add_sub_to_menu", add_sub_to_menu, 1 },
    { "add_embedded_entry", _wrap_add_embedded_entry, 1 },
    { "add_pattern_entry", _wrap_add_pattern_entry, 1 },
    { "add_syntax_entry", _wrap_add_syntax_entry, 1 },
