@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -49,11 +49,11 @@ void make_find_toolbar(void)
    find_combo_set_history(build_glist_from_file("find.hist", general_preferences.history));
    gtk_widget_show(find_combo);
    gtk_toolbar_append_space(GTK_TOOLBAR(findbar));
-   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_SEARCH, 21, 21);
+   pixmapwid = gnome_stock_pixmap_widget_at_size(window, GNOME_STOCK_PIXMAP_SEARCH, 24, 24);
    find_button = gtk_toolbar_append_item(GTK_TOOLBAR(findbar), 0, _("Find some text."), 0, pixmapwid, GTK_SIGNAL_FUNC(execute_find_cb), 0);
    gtk_widget_set_sensitive(find_button, FALSE);
    gtk_signal_connect(GTK_OBJECT(find_entry), "changed", GTK_SIGNAL_FUNC(check_find_button_cb), find_button);
-   gnome_app_add_docked(GNOME_APP(app), findbar, "FindBar", GNOME_DOCK_ITEM_BEH_NORMAL, GNOME_DOCK_TOP, 2, 6, 0);
+   gnome_app_add_docked(GNOME_APP(app), findbar, "FindBar", GNOME_DOCK_ITEM_BEH_NORMAL | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL, GNOME_DOCK_TOP, 2, 6, 0);
 }
 
 void execute_find_cb(GtkWidget* widget, gpointer data)
