@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -44,10 +44,14 @@ extern GtkWidget *hpaned1;
 extern GtkWidget *vpaned2;
 extern GtkWidget *hpaned3;
 extern GtkWidget *vpaned4;
+extern GtkWidget *panedbook1;
+extern GtkWidget *panedbook2;
+extern GtkWidget *panedbook3;
+extern GtkWidget *panedbook4;
+extern GHashTable *child_hash;
 extern GtkWidget *main_window_box;
 extern GtkWidget *edit_box;
 extern GtkWidget *files_book;
-extern GtkWidget *file_selector;
 extern GtkStyle *extext_style;
 extern GtkAccelGroup *accel_group;
 extern GnomePixmap *bookmark_pixmap;
