@@ -7,15 +7,19 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+
 #include "declarations.h"
 #include "mdi-routines.h"
+#include "file-io.h"
 #include "main.h"
 #include "menus.h"
 #include "misc.h"
@@ -29,16 +33,15 @@ void gds_file_ask_focus(GtkWidget *widget, GdsFile *gds_file);
 void undo_changed(GtkWidget *widget, GdsFile *gds_file);
 void editor_window_focused(GtkWidget *widget, GdkEventFocus *event, GdsFile *file);
 
-GdsFile *create_file(gchar *name)
+GtkStyle *normal_style = NULL;
+GtkStyle *changed_style = NULL;
+
+GdsFile *construct_file(GnomeVFSURI *uri)
 {
    GdsFile *new_file;
-   GtkWidget *label;
-   gchar cwd[256];
-   gchar *path;
-   gchar *file;
-   gchar *full;
    GdsFile *back_file;
 
+   if(!normal_style) init_label_styles(window->style);
    if(!general_preferences.use_gtk_theme)
    {
       extext_style->text[GTK_STATE_NORMAL] = general_preferences.fg;
@@ -48,16 +51,9 @@ GdsFile *create_file(gchar *name)
       extext_style->bg[GTK_STATE_INSENSITIVE] = general_preferences.gutter_bg;
       extext_style->bg_pixmap[GTK_STATE_NORMAL] = general_preferences.bg_pixmap ? general_preferences.bg_pixmap->pixmap : NULL;
    }
-   getcwd(cwd, 256);
-   path = get_path_from_filename(name);
-   file = get_file_from_filename(name);
-   if(!path) full = g_strconcat(cwd, "/", file, NULL);
-   else full = g_strconcat(path, "/", file, NULL);
-   new_file = (GdsFile *)gds_file_new(full, general_preferences.full_filenames);
-   if(!path)
-      new_file->props.dir = g_strdup(cwd);
-   else
-      new_file->props.dir = path;
+   
+   new_file = (GdsFile *)gds_file_new(uri, general_preferences.full_filenames);
+   new_file->props.dir = gnome_vfs_uri_get_host_path(uri);
    gtk_signal_connect(GTK_OBJECT(new_file->tear_away), "window_focused", GTK_SIGNAL_FUNC(file_change), new_file);
    gtk_signal_connect(GTK_OBJECT(new_file->tear_away), "attached", GTK_SIGNAL_FUNC(file_change), new_file);
    gtk_signal_connect(GTK_OBJECT(new_file->text), "undo_changed", GTK_SIGNAL_FUNC(undo_changed), new_file);
@@ -68,12 +64,6 @@ GdsFile *create_file(gchar *name)
    gtk_extext_set_use_tab_stops(GTK_EXTEXT(new_file->text), general_preferences.tab_stops);
    gtk_widget_show(GTK_WIDGET(new_file));
    files_list = g_list_append(files_list, (gpointer)new_file);
-   if(general_preferences.full_filenames)
-      label = gtk_label_new(full);
-   else
-      label = gtk_label_new(file);
-   gtk_widget_show(label);
-   g_free(full);
    total_files++;
    gtk_signal_connect(GTK_OBJECT(new_file->menu_item), "activate", GTK_SIGNAL_FUNC(gds_file_focus_with_raise), new_file);
    gtk_menu_append(GTK_MENU(docsmenu), new_file->menu_item);
@@ -81,14 +71,13 @@ GdsFile *create_file(gchar *name)
    new_file->props.auto_indent = general_preferences.auto_indent;
    new_file->props.use_spaces = general_preferences.use_spaces;
    new_file->props.spaces = general_preferences.spaces;
-   g_free(file);
    back_file = cur_file;
    cur_file = new_file;
 #ifdef WITH_PYTHON
    app_emit_scripting_signal("open-file");
 #endif
    cur_file = back_file;
-   gtk_notebook_append_page(GTK_NOTEBOOK(files_book), new_file->tear_away, label);
+   gtk_notebook_append_page(GTK_NOTEBOOK(files_book), new_file->tear_away, new_file->nb_label);
    gtk_widget_realize(new_file->text);
    gds_editor_set_style(GDS_EDITOR(new_file->text), extext_style);
    gtk_extext_set_draw_numbers(GTK_EXTEXT(new_file->text), general_preferences.draw_numbers);
@@ -98,6 +87,8 @@ GdsFile *create_file(gchar *name)
    gtk_extext_set_select_line(GTK_EXTEXT(new_file->text), general_preferences.select_line);
    cur_page = g_list_length(GTK_NOTEBOOK(files_book)->children) - 1;
    if(total_files > 1) gtk_notebook_set_page(GTK_NOTEBOOK(files_book), cur_page);
+   cur_file->changed_set = TRUE;
+   set_label_colors(new_file);
    return(new_file);
 }
 
@@ -150,27 +141,20 @@ void file_pos_changed(GtkWidget *widget, gint pos, gpointer data)
             gtk_widget_set_sensitive(search_menu_match, FALSE);
          else
             gtk_widget_set_sensitive(search_menu_match, TRUE);
+
+         cur_char = GTK_EXTEXT_INDEX(text, pos);
+         if(cur_char < 1 || !strchr("{[()]}", cur_char) || length < 1)
+            gtk_widget_set_sensitive(search_menu_select, FALSE);
+         else
+            gtk_widget_set_sensitive(search_menu_select, TRUE);
       }
       else
+      {
          gtk_widget_set_sensitive(search_menu_match, FALSE);
+         gtk_widget_set_sensitive(search_menu_select, FALSE);
+      }
       
-      if(GDS_EDITOR(cur_file->text)->changed && !cur_file->changed_set)
-      {
-         gchar *str;
-         gchar *changed;
-         if((str = strrchr(cur_file->filename, '/')) != NULL) { str++; } else { str = cur_file->filename; }
-         changed = g_strconcat(str, " *", NULL);
-         cur_file->changed_set = TRUE;
-         gds_file_set_title(cur_file, changed, general_preferences.full_filenames);
-         gtk_signal_connect(GTK_OBJECT(cur_file->menu_item), "activate", GTK_SIGNAL_FUNC(gds_file_focus_with_raise), cur_file);
-         g_free(changed);
-      }
-      else if(!GDS_EDITOR(cur_file->text)->changed && cur_file->changed_set)
-      {
-         cur_file->changed_set = FALSE;
-         gds_file_set_title(cur_file, cur_file->filename, general_preferences.full_filenames);
-         gtk_signal_connect(GTK_OBJECT(cur_file->menu_item), "activate", GTK_SIGNAL_FUNC(gds_file_focus_with_raise), cur_file);
-      }
+      set_label_colors(cur_file);
       set_length_depends(length);
    }
 }
@@ -222,3 +206,54 @@ void editor_window_focused(GtkWidget *widget, GdkEventFocus *event, GdsFile *fil
       if(file != cur_file) file_change(NULL, file);
    }
 }
+
+void init_label_styles(GtkStyle *style)
+{
+   int i;
+   if(!normal_style)
+   {
+      normal_style = gtk_style_copy(style);
+      gtk_style_ref(normal_style);
+      changed_style = gtk_style_copy(style);
+      gtk_style_ref(changed_style);
+   }
+   for(i = 0; i < 5; i++)
+   {
+      normal_style->fg[i].red = general_preferences.nb_normal.red;
+      normal_style->fg[i].green = general_preferences.nb_normal.green;
+      normal_style->fg[i].blue = general_preferences.nb_normal.blue;
+      changed_style->fg[i].red = general_preferences.nb_changed.red;
+      changed_style->fg[i].green = general_preferences.nb_changed.green;
+      changed_style->fg[i].blue = general_preferences.nb_changed.blue;
+   }
+   gdk_colormap_alloc_color(gdk_colormap_get_system (), &normal_style->fg[GTK_STATE_NORMAL], TRUE, TRUE);
+   gdk_colormap_alloc_color(gdk_colormap_get_system (), &changed_style->fg[GTK_STATE_NORMAL], TRUE, TRUE);
+}
+
+void destroy_label_styles(void)
+{
+   gtk_style_unref(normal_style);
+   gtk_style_unref(changed_style);
+}
+
+void set_label_colors(GdsFile *file)
+{
+   GdsEditor *editor;
+   GtkWidget *label;
+
+   editor = GDS_EDITOR(file->text);
+   if(editor->changed && !file->changed_set)
+   {
+      file->changed_set = TRUE;
+      label = GTK_BIN(file->menu_item)->child;
+      gtk_widget_set_style(label, changed_style);
+      gtk_widget_set_style(file->nb_label, changed_style);
+   }
+   else if(!editor->changed && file->changed_set)
+   {
+      file->changed_set = FALSE;
+      label = GTK_BIN(file->menu_item)->child;
+      gtk_widget_set_style(label, normal_style);
+      gtk_widget_set_style(file->nb_label, normal_style);
+   }
+}
