@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -49,6 +49,12 @@ gint get_which_radio_active(GSList *list);
 void set_which_radio_active(GSList *list, gint which);
 
 GList *build_file_list(void);
+
+gchar *localize_uri_string(gchar *string);
+
+void build_stock_pixmap(gchar *label, gchar **xpm_data, gint size);
+void modify_pixmap_state(GtkWidget *window, GtkStateType state, GnomePixmap **inout_pixmap);
+
 #ifdef __cplusplus
 }
 #endif
