@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -23,6 +23,7 @@
 #include "misc.h"
 #include "settings.h"
 #include "windows.h"
+#include "gtkextext/gtkextext.h"
 #include "widgets/gdsfile.h"
 
 static void add_manpage_string(GtkWidget *widget, gpointer data);
@@ -35,21 +36,6 @@ gboolean keep_open = FALSE;
 
 static void toggle_button_switch(GtkWidget *widget, gint *value);
 
-void match_bracket_cb(GtkWidget *widget, gpointer data)
-{
-   GtkExText *text;
-   gchar cur_char;
-   gint start = 0;
-
-   g_return_if_fail(cur_file != NULL);
-   text = GTK_EXTEXT(cur_file->text);
-   start = text->current_pos - 1;
-   cur_char = GTK_EXTEXT_INDEX(text, start);
-   if(!strchr("<{[()]}>", cur_char)) return;
-   if(find_bracket_match(cur_file->text, &start))
-      gtk_extext_set_position(text, start+1);
-}
-
 /* 
  * Find (F6)
  * If the file has text in the filename_entry, look for it.
@@ -160,7 +146,7 @@ gboolean find_text(GtkExText *text, gint start, gint end, gchar *search_string,
    g_return_val_if_fail(text != NULL, FALSE);
    if(is_regex)
    {
-      if(gds_editor_compile_regex(search_string, &regex))
+      if(gtk_extext_compile_regex(search_string, &regex))
       {
          pos = gtk_extext_regex_search(text, start, &regex, TRUE, match);
          if(pos == -1) pos = 0;
@@ -288,7 +274,7 @@ gint replace_all_text(GtkExText *text, gchar *search_text, gchar *new_text, gboo
          end += (strlen(new_text) - strlen(search_text));
          replacements++;
       }
-      else if(search_multi && end != GTK_EXTEXT(cur_file->text)->length)
+      else if(search_multi)
       {
          GdsFile *file;
          gint total;
@@ -381,7 +367,7 @@ void find_line_cb(GtkWidget *widget, GnomeGotoLineDialog *gtl)
    if(line_num > -1 && line_num <= GTK_EXTEXT(cur_file->text)->line_count)
    {
       text = gtk_entry_get_text(GTK_ENTRY(gtl->line_entry));
-      gtk_extext_set_line(GTK_EXTEXT(cur_file->text), line_num-1);
+      gtk_extext_set_line(GTK_EXTEXT(cur_file->text), line_num);
       gnome_goto_line_dialog_add_search_string(gtl, text);
       line_combo_add_string(text);
       gtk_entry_set_text(GTK_ENTRY(line_entry), "");
@@ -541,6 +527,39 @@ void next_bookmark_cb(GtkWidget *widget, gpointer data)
    else display_message("No more bookmarks.", FLASH);
 }
 
+void match_bracket_cb(GtkWidget *widget, gpointer data)
+{
+   GtkExText *text;
+   gchar cur_char;
+   gint start = 0;
+
+   g_return_if_fail(cur_file != NULL);
+   text = GTK_EXTEXT(cur_file->text);
+   start = text->current_pos - 1;
+   cur_char = GTK_EXTEXT_INDEX(text, start);
+   if(!strchr("<{[()]}>", cur_char)) return;
+   if(find_bracket_match(cur_file->text, &start))
+      gtk_extext_set_position(text, start+1);
+}
+
+void select_to_bracket_cb(GtkWidget *widget, gpointer data)
+{
+   GtkExText *text;
+   gchar cur_char;
+   gint start = 0;
+   gint end = 0;
+   g_return_if_fail(cur_file != NULL);
+   text = GTK_EXTEXT(cur_file->text);
+   start = end = text->current_pos;
+   cur_char = GTK_EXTEXT_INDEX(text, start);
+   if(!strchr("{[()]}", cur_char)) return;
+   if(find_bracket_match(cur_file->text, &end))
+   {
+      gtk_extext_select_region(text, start < end ? start : end, end > start ? end+1 : start+1);
+   }
+
+}
+
 static void toggle_button_switch(GtkWidget *widget, gint *value)
 {
    *value = *value ? 0 : 1;
