@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -22,14 +22,14 @@ extern "C" {
 #endif
 
 #include "declarations.h"
-#include "widgets/gnomefilelist.h"
+#include "widgets/gnome-file-selector.h"
 
 guint get_last_session(void);
 void save_session_cb(GtkWidget *widget, gpointer data);
 void open_session_cb(GtkWidget *widget, gpointer data);
-void open_session_continue(GtkWidget *widget, GnomeFileList *file_list);
+void open_session_continue(GtkWidget *widget, GnomeFileSelector *file_list);
 void save_session_as_cb(GtkWidget *widget, gpointer data);
-void save_session_as_continue(GtkWidget *widget, GnomeFileList *file_list);
+void save_session_as_continue(GtkWidget *widget, GnomeFileSelector *file_list);
 void save_session_as_error_continue(GtkWidget *widget, gchar *filename);
 gboolean save_session_to_file(gchar *filename);
 guint open_session_from_file(gchar *filename);
