@@ -1,4 +1,4 @@
-/*  signals.h
+/*  Glimmer - signals.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -7,7 +7,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,12 +21,18 @@
 extern "C" {
 #endif
 
+#ifdef WITH_PYTHON
+
+#include "declarations.h"
+#include "widgets/gdsfile.h"
+
 gboolean table_emit_scripting_signal(gchar *signal, GdsFileHighlightTable *table);
 gboolean file_emit_scripting_signal(gchar *signal, GdsFile *file);
 gboolean app_emit_scripting_signal(gchar *signal);
 
 gboolean call_scripting_function(gpointer function);
 gboolean check_scripting_function(gpointer function);
+gboolean call_scripting_function_extended(gpointer function, gint control, gint shift, gint alt, gint key, gint pos);
 
 void remove_signal_hooks(GdsFileHighlightTable *table);
 
@@ -39,6 +45,7 @@ extern GList *change_signals;
 
 extern GList *move_left_signals;
 extern GList *move_right_signals;
+#endif
 
 #ifdef __cplusplus
 }
